/*HEADER------------------------------------------------------------------------
|
| PROGRAM:  create_cox_reg_rpt.sas (macro)
||
| PURPOSE:  This macro creates report using output from the macro %distributed_regression 
                        for Cox regression . 
                        It can also be used to generate similar reports using output from macro cox_reg_with_data_input. 
                       The later macro uses standard
                        SAS regression procedure (PHREG) on combined dataset and generate output datasets that follow 
                        naming conventions of the datasets generated by  %distributed_regression (up to a prefix).
                    |
|
| MACRO PARAMETERS:
|
	 INPUT:    
          lib_in                       The name of the library that has output datasets from regression.
                                          For DRA it is always msoc library. Must be specified.

         rpt_dir_out			   The name of the output  directory for the report

         prefix_r					  Prefix used for dataset created by regression macro.
                                          For example, &RunID_.
|	     
		distributed_reg_in		This affects the name of the report. When distributed_reg_in=1
                                           the report name includes word Distributed.

     rpt_format						File format for the report. Can be PDF or RTF.
                                          The default is PDF

      pdf_style                     Determines ODS style used with the report.
                                         Default value produces calor report. For black and
                                         white report use style=JOURNAL,JOURNAL2  
                                          Can be used not only with PDF but with RTF
                                          output as well.

      format_N                     Numeric format used in printed tables.
                                          Default is D11.3.

    Output:                          Report in pdf or rtf format in the directory specified by
                                          by the parameter rpt_dir_out.
                                          For example, dc1_Cox_Distributed_reg_rpt.rtf

   	USAGE EXAMPLES:
 
		    %LET prefix_c=dc2_; 
		   %PUT prefix_c=&prefix_c.;

		   %create_cox_reg_rpt( lib_in=msoc
                                            ,rpt_dir_out=&dplocal.
											,strata_vars=dp_cd
                                             ,prefix_r=&prefix_c.
                                            ,distributed_reg_in=1
									         ,pdf_style=JOURNAL2						
								             ,rpt_format=rtf
									         ,format_N=D12.4
                                             );
	
	First version:  02/06/2017, Yury Vilk. 
+------------------------------------------------------------------------HEADER*/	 
 
 %MACRO create_cox_reg_rpt( lib_in=msoc
                                         , rpt_dir_out=./
										 ,strata_vars=
                                         ,prefix_r=
                                        , distributed_reg_in=1
									    ,pdf_style=DEFAULT
									   ,bradiusmax=2%
									    ,bradiusmin=0.5%
									   ,additional_title=
									   ,format_N=D12.4
                                       ,cnt_indep_vars_sh=10
									    ,rpt_format=pdf
                                         );

 %LOCAL regr_type_dsc distributed_reg_dsc _vars_for_modelfit 
               _indep_vars _indep_vars_sh  _indep_vars_numb  _auc_
			    graph_grp dash min_Mean max_Mean  baseline_est __attrid_option
				 dependent_vars	 
        ;


/*
 %LET lib_in=dplocal;
 %LET rpt_dir_out=&dplocal.;
 %LET prefix_r=cc1_;
 %LET distributed_reg_in=0;
  %LET format_N=D12.4;
 * %LET format_N=E17.;
  %LET cnt_indep_vars_sh=10;
  *%LET pdf_style=JOURNAL; 
 %LET pdf_style=DEFAULT;
*/

/* 
%LET lib_in=msoc;
 *%LET rpt_dir_out=&msoc.;
  %LET rpt_dir_out=&dplocal.;
 %LET prefix_r=dc3_;
 * %LET prefix_r=dr1_;
 %LET distributed_reg_in=1;
  %LET format_N=D12.4 ;
%LET additional_title=;
 %LET cnt_indep_vars_sh=10;
		*%LET strata_vars=;

   %LET strata_vars=dp_cd;
 * %LET strata_vars=strata_var2;
*  %LET strata_vars=dp_cd strata_var2;

   * %LET pdf_style=JOURNAL;  
 %LET pdf_style=DEFAULT;

	 %LET pdf_style=DEFAULT;
	 %LET bradiusmax=2%	 ;
     %LET bradiusmin=0.5%;
 */

 %LET regr_type_dsc=Cox;
 %LET distributed_reg_dsc=Distributed;
  %LET graph_grp=%STR(group=dp_cd);
  %LET dash=_;
  %LET additional_title=;
  %LET group_strata_vars=;
  %LET count_strata_vars=0;
   /* For buble graph*/
  %LET  __attrid_option=;

   %LET NOTOC=NOTOC;
  

 %MACRO wrapper;

%IF &distributed_reg_in.=0 %THEN   
				   %DO;
       %LET distributed_reg_dsc=;
	 /*  %LET graph_grp=; */
	   %LET dash=;
					%END;
 %IF %LENGTH(&strata_vars.) >0 %THEN
	                     %DO;
	 %LET strata_vars=%SYSFUNC(COMPBL(&strata_vars.));
	 %LET count_strata_vars=%SYSFUNC(COUNTW(&strata_vars.,%STR( )));;
	%IF &count_strata_vars. =1 %THEN %LET group_strata_vars=%STR(/GROUP=&strata_vars.);	
	%ELSE											     %LET group_strata_vars=%STR(/GROUP=Stratum);	
			
						  %END;

	/* Optimize bubble graph for black and white printer when ODS style is JOURNAL, JORNAL2 ...	*/

	 %IF %SUBSTR(&pdf_style., 1,5)=JOURN %THEN 	%LET __attrid_option=%STR(ATTRID=BW);

   	%IF &rpt_format.=rtf %THEN %LET NOTOC=;
 %MEND;

 %wrapper;

 %LET title1=&regr_type_dsc. &distributed_reg_dsc. Regression. &additional_title.;
 %PUT title1=&title1.;
 %PUT count_strata_vars=&count_strata_vars.;
  %PUT group_strata_vars=&group_strata_vars.;
  %PUT __attrid_option=&__attrid_option.;

  
 DATA _NULL_;
    SET  &lib_in..&prefix_r.MODELINFO;
	   IF LOWCASE(Description)="dependent variable";
	   CALL SYMPUT("dependent_vars",STRIP(cValue)); 
  RUN;

  %PUT dependent_vars=&dependent_vars.;

 PROC FORMAT;
   VALUE  _Total
       .="Total";
 RUN;
 QUIT;
	
	DATA _NULL_;
		LENGTH _indep_vars $ 32000;
		 RETAIN _indep_vars;
		   SET &lib_in..&prefix_r.p_est END=last;
			 _indep_vars=CATX(" ",_indep_vars,Variable);

			 IF _N_=&cnt_indep_vars_sh. THEN 	CALL SYMPUT("_indep_vars_sh", STRIP( _indep_vars));

			 IF last THEN  
			                    DO;
               CALL SYMPUT("_indep_vars", STRIP( _indep_vars));
			   CALL SYMPUT("_indep_vars_numb", STRIP( PUT(_N_,8.)));
			  IF _N_ < 10 THEN 	CALL SYMPUT("_indep_vars_sh", STRIP( _indep_vars));
			                   END;

		RUN;

		%PUT _indep_vars=&_indep_vars.	;
		%PUT _indep_vars_sh=&_indep_vars_sh.	;
		%PUT _indep_vars_numb=&_indep_vars_numb.	;
   
  	 ODS GRAPHICS ON /  RESET=INDEX 
                                         IMAGENAME="&prefix_r.D_Reg" /*To see default image names use ODS TRACE */
                                         BORDER=OFF
									      IMAGEFMT=PNG
                                          ;

	   OPTIONS DEVICE=PNG;

	 ODS LISTING CLOSE	;

 ODS &rpt_format. DPI=400 FILE="&rpt_dir_out.&prefix_r.&regr_type_dsc._&distributed_reg_dsc.&dash.reg_rpt.&rpt_format." 
						       &NOTOC. STYLE=&pdf_style. NOGTITLE;

     OPTIONS ORIENTATION=LANDSCAPE;
	 *OPTIONS ORIENTATION=PORTRAIT;
		OPTIONS MISSING=" ";

	  TITLE1 &title1;

	 
TITLE2; 

            PROC REPORT DATA=&lib_in..&prefix_r.MODELINFO;
			COLUMN  ("Model Information" (Description cValue) );
			  DEFINE Description/DISPLAY;
			  LABEL  cValue="Value";
	     	RUN;
			QUIT;

			DATA _NULL_;
			  SET &lib_in..&prefix_r.MODELINFO;
			  IF LOWCASE(STRIP(Description))="ties handling" ;
			  IF LOWCASE(STRIP(cValue))="breslow" 
             THEN CALL SYMPUT("baseline_est","Breslow");
			 ELSE  CALL SYMPUT("baseline_est","Fleming-Harrington ");
			RUN;

				ODS STARTPAGE=NO;

				 %IF %LENGTH(&strata_vars.) =0 %THEN
	                     %DO;	
		   PROC REPORT DATA=&lib_in..&prefix_r.CENS_SUM;
		     COLUMN  ("Summary of the Number of Event and Censored Values" 
                                 (&strata_vars. Total  Event  Censored  PctCens) 
                                );
			  LABEL  PctCens="Percent Censored";
	     	RUN;
			QUIT;
			
						  %END;
			  %ELSE
		             %DO;
	
		   PROC REPORT DATA=&lib_in..&prefix_r.CENS_SUM;
		     COLUMN  ("Summary of the Number of Event and Censored Values" 
                                 (Stratum &strata_vars. Total  Event  Censored  PctCens) 
                                );
			  DEFINE Stratum/DISPLAY;
			  LABEL  PctCens="Percent Censored";
			  Format Stratum _TOTAL.;
	     	RUN;
			QUIT;
					%END;

		PROC REPORT DATA=&lib_in..&prefix_r.Convrg_Status ;
		   COLUMN  Reason;
			 * DEFINE Reason/DISPLAY;
		   FORMAT Reason $100. ;
			LABEL Reason="Convergence Message";
		RUN;
	   QUIT;

		 ODS STARTPAGE=NO;
	
		 /*%LET format_N=D12.4;
		  %LET format_N=E17.;
		 */
		  
		PROC REPORT DATA = &lib_in..&prefix_r.Iter_Parms_Hist ;
		   COLUMN  ("Iteration history. First &cnt_indep_vars_sh. columns shown." (Iteration  LogLikelihood &_indep_vars_sh.) );
			  DEFINE Iteration/ORDER;
			LABEL  LogLikelihood="Log Likelihood";
				FORMAT LogLikelihood &_indep_vars_sh. &format_N. ;  
		RUN;
	   QUIT;

	   ODS STARTPAGE=YES;

	 PROC REPORT DATA=  &lib_in..&prefix_r.modelfit ;
		   COLUMN  ("Model Fit Statistics" (Criterion WithoutCovariates WithCovariates) );
			  DEFINE Criterion/ DISPLAY;
			  	FORMAT WithoutCovariates WithCovariates  &format_N. ;
				LABEL WithoutCovariates="Without Covariates" 
						    WithCovariates="With Covariates"
				   		 ;
		RUN;
	     QUIT;


	  ODS STARTPAGE=NO;
	 PROC REPORT DATA=  &lib_in..&prefix_r.GLOB_NULL_CHISQ ;
		   COLUMN  ("Testing Global Null Hypothesis: BETA=0" (Test ChiSq DF ProbChiSq) );
			  DEFINE Test/ DISPLAY;
			  WHERE UPCASE(Test)="LIKELIHOOD RATIO";
			FORMAT ChiSq  &format_N.  ProbChiSq PValue6.4;
			LABEL ChiSq="Chi-Square"
                       ProbChiSq="Pr > Chi-Square"
                          ;
		RUN;
	    QUIT;

		ODS STARTPAGE=NO;

		TITLE2 ;
		 FOOTNOTE1 ;

	PROC REPORT DATA=  &lib_in..&prefix_r.P_EST(RENAME=(StdErr=StdErr1 ));
	 COLUMN  ("Analysis of Maximum Likelihood Estimates" (Variable DF Estimate StdErr1 
                                                                                               ChiSq ProbChiSq  
                                                                                               HazardRatio HR_LowerCL HR_UpperCL ) );
			  DEFINE Variable/ DISPLAY;
		FORMAT Estimate StdErr1 ChiSq HazardRatio &format_N.  ProbChiSq PValue6.4;
              LABEL  Variable="Parameter"
			                Estimate="Parameter Estimate"
			               StdErr1="Standard Error"
                           ChiSq="Chi-Square"
                           ProbChiSq="Pr > Chi-Square"
						   HazardRatio="Hazard Ratio"
                          ; 
		RUN;
	  QUIT;

		ODS STARTPAGE=NO;

		DATA &prefix_r.COV_EST_rpt;
		   SET &lib_in..&prefix_r.COV_EST;
		    IF _N_ <= &cnt_indep_vars_sh.;
		RUN;

		PROC REPORT DATA=&prefix_r.COV_EST_rpt;
	        COLUMN  ("Covariance of Estimates. First &cnt_indep_vars_sh. columns shown." (Variable  &_indep_vars_sh.) );
			  DEFINE Variable/ DISPLAY;	
           FORMAT _NUMERIC_ &format_N.; 
		RUN;

											
	 PROC SGPLOT DATA=&lib_in..&prefix_r.baseln_hazard
								CYCLEATTRS ASPECT=1	 
								;
		TITLE2 "Baseline cumulative hazard function (&baseline_est. estimator)";	
	   *  SCATTER X=&dependent_vars.  Y=_Cum_Hazard0_ &group_strata_vars.;
		 LOESS X=&dependent_vars.  Y=_Cum_Hazard0_ &group_strata_vars.;
	
       LABEL _Cum_Hazard0_="Baseline cumulative hazard"
				  &dependent_vars.="&dependent_vars. (Events time)"
                 ;	
      RUN;
	  QUIT;

	  
   TITLE2 " ";
   TITLE3 " ";

   
 TITLE2  "Survival function calulated at covariates mean" ;

PROC SGPLOT DATA=&lib_in..&prefix_r.baseln_survival	 CYCLEATTRS	;
 	  LOESS Y= _SURVIVAL_at_MEAN	 X=&dependent_vars. &group_strata_vars.;
	 	YAXIS MIN=0 MAX=1;
		 LABEL _SURVIVAL_at_MEAN="Survival function " 
					   &dependent_vars.="&dependent_vars. (Events time)"
       ;
   RUN;
 
   TITLE2 " ";
   TITLE3 " ";
 
 PROC SQL;
   CREATE TABLE _all_dp_cd AS
     SELECT DISTINCT dp_cd
	   FROM   &lib_in..&prefix_r.RESID_SUM_BY_PCT
	   ORDER BY dp_cd
          ;
	QUIT;

 	  
 data __attrmap_2;
    SET _all_dp_cd;
  id='BW'; value=dp_cd;
	 /*For first 3 dp_cd set colors that look clearly different when printed on black and white printer. 
       This will be used with black and with ODS style like JOURNAL */

   IF _N_ =1 THEN DO;fillcolor='black'; output; END;
   IF  _N_ =2 THEN DO; fillcolor='grey'; output; END;
   IF  _N_ =3 THEN DO; fillcolor='white'; output; END;
				/*Use default colors for other values*/
  run;
 
	DATA &prefix_r.RESID_SUM_BY_PCT_g	;
	   SET &lib_in..&prefix_r.RESID_SUM_BY_PCT;

			   IF dp_cd >0;

		LABEL _XBETA__Mean="Mean of Linear Predictor (Beta*Z)"
 				    Resid_Martingale_Mean="Martingale Residual"
				   Resid_Dev_Mean="Deviance Residual"			
			  ;
	RUN;

	PROC SORT DATA=&prefix_r.RESID_SUM_BY_PCT_g;
    	BY dp_cd _XBETA__Mean ;
	RUN;

    PROC MEANS DATA=&prefix_r.RESID_SUM_BY_PCT_g	NOPRINT;
		  FREQ 	_FREQ_;
		  VAR Resid_Dev_Mean	;
		 OUTPUT 	 OUT=&prefix_r.Resid_Dev_Mean(DROP=_TYPE_ _FREQ_)
						   MEAN(Resid_Dev_Mean)=Resid_Dev_Mean_over_all;
		 RUN;

		DATA _NULL_;
		  SET &prefix_r.Resid_Dev_Mean;
		   CALL SYMPUT("Resid_Dev_Mean_over_all",STRIP(PUT(Resid_Dev_Mean_over_all,12.8 )));
		RUN;

		%PUT  Resid_Dev_Mean_over_all=&Resid_Dev_Mean_over_all.;

	 
    PROC SGPLOT DATA=&prefix_r.RESID_SUM_BY_PCT_g	
								CYCLEATTRS	ASPECT=1   DATTRMAP=__attrmap_2
								;
	    TITLE2 "Martingale Residual vs. mean of Linear Predictor.";
		TITLE3 "The data are grouped by percentiles of linear predictor values."  ;
        TITLE4 "The radius of a bubble is proportional to a number of observations in a group.";

   BUBBLE X=_XBETA__Mean Y=Resid_Martingale_Mean  SIZE= _FREQ_ / &graph_grp. 
                                                                                                          BRADIUSMAX=&BRADIUSMAX.
																										  &__attrid_option.
                                            ;
     	REFLINE   0  /AXIS=Y; 	
   RUN;

    TITLE2 ;
    TITLE3 ;
    TITLE4;

    PROC SGPLOT DATA=&prefix_r.RESID_SUM_BY_PCT_g	
								CYCLEATTRS	ASPECT=1  DATTRMAP=__attrmap_2
								;
	    TITLE2 "Deviance Residual vs. mean of Linear Predictor.";
		TITLE3 "The data are grouped by percentiles of linear predictor values."  ;
        TITLE4 "The radius of a bubble is proportional to a number of observations in a group.";

   BUBBLE X=_XBETA__Mean Y=Resid_Dev_Mean SIZE= _FREQ_ / &graph_grp.
                                                                                                          BRADIUSMAX=&BRADIUSMAX.
																										  &__attrid_option.
                                            ;
     	REFLINE   &Resid_Dev_Mean_over_all.  /AXIS=Y; 	
   RUN;

    TITLE2 ;
    TITLE3 ;
    TITLE4;

	  ODS &rpt_format. CLOSE;

        ODS LISTING;

	%MEND;	



	







	



	      